function [sfcstPtwise_stat, sfcstAvg_stat, dfcst_stat, param_est] = ...
    pool_linear(Y,X,draws,param0,save_file)
warn2err = warning('error','MATLAB:nearlySingularMatrix');
warn2err2 = warning('error','MATLAB:illConditionedMatrix');

% save_file{1} nonempty: save MCMC details
    % save_file{3} = true: save draws needed for fcst only
    % save_file{3} = false: save draws needed for both fcst and 
    %                       recovering underlying dist
% save_file{1} empty: not save MCMC details
% save_file{2}: sample and estimator

%% Input: Y
YY = Y(1:end-1,:);
[T1,N] = size(YY);
T = T1-1;
ix0 = find(Y(end,:)==0);
ix1 = find(Y(end,:)~=0);

%% Input: X
XX = X(2:end-1,:,:);
n_coef = size(XX,3)+1;
Xs = nan(T,N,n_coef);
if ~isempty(X)
    Xs(:,:,2:n_coef) = XX;
end

%% Input: numbers of draws
B_draw = draws(1);
N_draw = draws(2);

%% Input: tuning parameters for prior
ttau = param0{2};

%% Prepare: prior parameters 
% rho, sigma2
M_coef0 = zeros(n_coef+1,1);
V_coef0 = ttau(1)*eye(n_coef+1);
a_ssigma20 = 3;
b_ssigma20 = mean(var(YY,1))*(a_ssigma20-1);

%% Prepare: initial values 
% y*_it
YY_star = YY;

% rho, lambda, sigma2
Ys = YY_star(2:end,:);
Xs(:,:,1) = YY_star(1:end-1,:);

vecX = [reshape(Xs,N*T,n_coef) ones(N*T,1)];
M_coef1 = (vecX'*vecX)\(vecX'*Ys(:));
rrho = M_coef1(1:n_coef);
llambda = M_coef1(end);
ssigma2 = var(Ys(:)-vecX*M_coef1);

%% Prepare: save draws
m_draw = nan(B_draw+N_draw,N);
s_draw = nan(B_draw+N_draw,N);

param_draw = nan(B_draw+N_draw,n_coef+2);

%% Gibbs sampler
for i_draw = 1:(B_draw+N_draw)
    %% Display iteration
    if mod(i_draw,100) == 0
        disp([save_file{2} ': i_draw = ' num2str(i_draw)])
    end
    
    %% Draw rho, lambda, sigma2 
	Ys = YY_star(2:end,:);
    Xs(:,:,1) = YY_star(1:end-1,:);
 
    vecX = [reshape(Xs,N*T,n_coef) ones(N*T,1)];
    V_coef1 = eye(n_coef+1)/(vecX'*vecX/ssigma2+eye(n_coef+1)/V_coef0);
    M_coef1 = V_coef1*(vecX'*Ys(:)/ssigma2+V_coef0\M_coef0);
    coef1 = chol(V_coef1)'*randn(n_coef+1,1)+M_coef1;
    rrho = coef1(1:n_coef);
    llambda = coef1(end);
    
    a_ssigma21 = a_ssigma20+N*T/2;
    b_ssigma21 = b_ssigma20+(Ys(:)-vecX*coef1)'*(Ys(:)-vecX*coef1)/2;
    ssigma2 = 1/gamrnd(a_ssigma21,1/b_ssigma21);
    
    %% Prepare: forecast evaluation
    if ~isempty(X)
        m_draw(i_draw,:) = ([YY_star(end,:)' reshape(X(end,:,:),N,n_coef-1)]*rrho)'+llambda;
    else
        m_draw(i_draw,:) = (YY_star(end,:)'*rrho)'+llambda;
    end
    s_draw(i_draw,:) = sqrt(ssigma2);
        
    %% Save draws
    param_draw(i_draw,:) = [rrho' llambda ssigma2];   
end

%% Forecast evaluation
fcst_eval;
    
%% Common parameter estimates
param_est= reshape([mean(param_draw(ix_out,:)); quantile(param_draw(ix_out,:),[.05 .95]')],1,[]);

%% Save MCMC details if needed
if ~isempty(save_file{1}) 
    parsave([save_file{1} save_file{2} '.mat'],param_draw)   
end

warning(warn2err);
warning(warn2err2);