function [sfcstPtwise_stat, sfcstAvg_stat, dfcst_stat, param_est] = ...
    pool_tobit(Y,X,draws,param0,save_file)
warn2err = warning('error','MATLAB:nearlySingularMatrix');
warn2err2 = warning('error','MATLAB:illConditionedMatrix');

% save_file{1} nonempty: save MCMC details
    % save_file{3} = true: save draws needed for fcst only
    % save_file{3} = false: save draws needed for both fcst and 
    %                       recovering underlying dist
% save_file{1} empty: not save MCMC details
% save_file{2}: sample and estimator

%% Input: Y
YY = Y(1:end-1,:);
[T1,N] = size(YY);
T = T1-1;
ix0 = find(Y(end,:)==0);
ix1 = find(Y(end,:)~=0);

%% Input: X
XX = X(2:end-1,:,:);
n_coef = size(XX,3)+1;
Xs = nan(T,N,n_coef);
if ~isempty(X)
    Xs(:,:,2:n_coef) = XX;
end

%% Input: numbers of draws
B_draw = draws(1);
N_draw = draws(2);

%% Input: tuning parameters for prior
ttau = param0{2};

%% Prepare: prior parameters 
% p(y*_i0)
m_my0_0 = 0;
v_my0_0 = ttau(2);
a_vy0_0 = 3;
b_vy0_0 = 2*ttau(3);

% rho, sigma2
M_coef0 = zeros(n_coef+1,1);
V_coef0 = ttau(1)*eye(n_coef+1);
a_ssigma20 = 3;
b_ssigma20 = mean(var(YY,1))*(a_ssigma20-1);

%% Prepare: initial values 
% y*_it
YY_star = YY;

% rho, lambda, sigma2
Ys = YY_star(2:end,:);
Xs(:,:,1) = YY_star(1:end-1,:);

vecX = [reshape(Xs,N*T,n_coef) ones(N*T,1)];
M_coef1 = (vecX'*vecX)\(vecX'*Ys(:));
rrho = M_coef1(1:n_coef);
llambda = M_coef1(end);
ssigma2 = var(Ys(:)-vecX*M_coef1);

%% Prepare: save draws
m_draw = nan(B_draw+N_draw,N);
s_draw = nan(B_draw+N_draw,N);

param_draw = nan(B_draw+N_draw,n_coef+2);

if ~isempty(save_file{1})  
    if ~save_file{3}
        YYT_star_draw = nan(B_draw+N_draw,N);   
    else
        YY_star_draw = nan(B_draw+N_draw,T1,N);
        mm_y0_draw = nan(B_draw+N_draw,1);
        vv_y0_draw = nan(B_draw+N_draw,1);
    end
end

%% Prepare: segment structure
seg_start = cell(1,N);
seg_end = cell(1,N);
n_seg = nan(1,N);
for i = 1:N
    seg_all = diff(YY(:,i)==0);
    seg_start{i} = find(seg_all==1)+1;
    seg_end{i} = find(seg_all==-1);
    if YY(1,i)==0
        seg_start{i} = [1; seg_start{i}];
    end
    if YY(end,i)==0
        seg_end{i} = [seg_end{i}; T1];
    end
    n_seg(i) = length(seg_start{i});
end

%% Gibbs sampler
for i_draw = 1:(B_draw+N_draw)
    %% Display iteration
    if mod(i_draw,100) == 0
        disp([save_file{2} ': i_draw = ' num2str(i_draw)])
    end
    
    %% Draw p(y*_i0)    
    v_my0_1 = 1/(N+1/v_my0_0);
    m_my0_1 = (sum(YY_star(1,:))+m_my0_0/v_my0_0)*v_my0_1;
    a_vy0_1 = a_vy0_0+N/2;
    b_vy0_1 = b_vy0_0+(sum(YY_star(1,:).^2)+m_my0_0^2/v_my0_0-m_my0_1^2/v_my0_1)/2;
    vv_y0 = 1/gamrnd(a_vy0_1,1/b_vy0_1);
    mm_y0 = sqrt(v_my0_1*vv_y0)*randn(1)+m_my0_1;
    
    %% Draw y*_it    
    DD_T1 = toeplitz(rrho(1).^(0:(T1-1)),[1 zeros(1,T1-1)]);
    
    QQ_T1 = diag((1-rrho(1).^((1:T1)*2))/(1-rrho(1)^2));
    for t = 1:T1
        QQ_T1(t,(t+1):T1) = rrho(1).^(1:(T1-t))*QQ_T1(t,t);
        QQ_T1((t+1):T1,t) = QQ_T1(t,(t+1):T1);
    end
    
    QQ_T1_1 = diag([0 (1-rrho(1).^((1:(T1-1))*2))/(1-rrho(1)^2)] ...
        +vv_y0/ssigma2*rrho(1).^((0:(T1-1))*2)); % for y_i0 = 0
    for t = 1:T1
        QQ_T1_1(t,(t+1):T1) = rrho(1).^(1:(T1-t))*QQ_T1_1(t,t);
        QQ_T1_1((t+1):T1,t) = QQ_T1_1(t,(t+1):T1);
    end
            
    for i = 1:N
        if n_seg(i) == 0
            continue
        end
        for i_seg = 1:n_seg(i)
            
            if seg_end{i}(i_seg) == T1
                TT = seg_end{i}(i_seg)-seg_start{i}(i_seg)+1;
            else
                TT = seg_end{i}(i_seg)-seg_start{i}(i_seg)+2;
            end
                        
            if seg_start{i}(i_seg) == 1
                if ~isempty(X)
                    AA = (1-rrho(1).^((1:TT)-1)')/(1-rrho(1))*llambda ...
                        +DD_T1(1:TT,1:TT)*[mm_y0; reshape(XX(seg_start{i}(i_seg)-1+(1:(TT-1)),i,:), ...
                                                          TT-1,n_coef-1)*rrho(2:end)];
                else
                    AA = (1-rrho(1).^((1:TT)-1)')/(1-rrho(1))*llambda ...
                        +DD_T1(1:TT,1:TT)*[mm_y0; zeros(TT-1,1)];
                end
                QQ = QQ_T1_1(1:TT,1:TT);
            else
                if ~isempty(X)
                    AA = (1-rrho(1).^(1:TT)')/(1-rrho(1))*llambda ...
                        +rrho(1).^(1:TT)'*YY(seg_start{i}(i_seg)-1,i) ...
                        +DD_T1(1:TT,1:TT)*reshape(XX(seg_start{i}(i_seg)-1+(0:(TT-1)),i,:), ...
                                                  TT,n_coef-1)*rrho(2:end);
                else
                    AA = (1-rrho(1).^(1:TT)')/(1-rrho(1))*llambda ...
                        +rrho(1).^(1:TT)'*YY(seg_start{i}(i_seg)-1,i);
                end
                QQ = QQ_T1(1:TT,1:TT);
            end
            
            if seg_end{i}(i_seg) ~= T1
                AA = AA(1:(TT-1))+QQ(1:(TT-1),TT)/QQ(TT,TT)*(YY(seg_end{i}(i_seg)+1,i)-AA(TT));
                QQ = QQ(1:(TT-1),1:(TT-1))-QQ(1:(TT-1),TT)*QQ(TT,1:(TT-1))/QQ(TT,TT);
                TT = TT-1;
            end
            
            if TT == 1
                YY_star(seg_start{i}(i_seg),i) = AA+sqrt(QQ)*trandn(-inf,-AA/sqrt(QQ));
            else
                for t = 1:TT
                    aux_A = AA;
                    aux_A(t) = [];
                    aux_Ystar = YY_star(seg_start{i}(i_seg):seg_end{i}(i_seg),i);
                    aux_Ystar(t) = [];
                    
                    aux_Q = QQ;
                    aux_Q(t,:) = [];
                    aux_v = aux_Q(:,t);
                    aux_Q(:,t) = [];
                    
                    aux_m = AA(t)+(aux_v'/aux_Q)*(aux_Ystar-aux_A);
                    aux_s = sqrt(QQ(t,t)-(aux_v'/aux_Q)*aux_v)*sqrt(ssigma2);
                    YY_star(seg_start{i}(i_seg)-1+t,i) = aux_m+aux_s*trandn(-inf,-aux_m/aux_s);
                end
            end
        end
    end
    
    %% Draw rho, lambda, sigma2 
	Ys = YY_star(2:end,:);
    Xs(:,:,1) = YY_star(1:end-1,:);
 
    vecX = [reshape(Xs,N*T,n_coef) ones(N*T,1)];
    V_coef1 = eye(n_coef+1)/(vecX'*vecX/ssigma2+eye(n_coef+1)/V_coef0);
    M_coef1 = V_coef1*(vecX'*Ys(:)/ssigma2+V_coef0\M_coef0);
    coef1 = chol(V_coef1)'*randn(n_coef+1,1)+M_coef1;
    rrho = coef1(1:n_coef);
    llambda = coef1(end);
    
    a_ssigma21 = a_ssigma20+N*T/2;	    
    b_ssigma21 = b_ssigma20+(Ys(:)-vecX*coef1)'*(Ys(:)-vecX*coef1)/2;
    ssigma2 = 1/gamrnd(a_ssigma21,1/b_ssigma21);
    
    %% Prepare: forecast evaluation
    if ~isempty(X)
        m_draw(i_draw,:) = ([YY_star(end,:)' reshape(X(end,:,:),N,n_coef-1)]*rrho)'+llambda;
    else
        m_draw(i_draw,:) = (YY_star(end,:)'*rrho)'+llambda;
    end
    s_draw(i_draw,:) = sqrt(ssigma2);
    
    %% Save draws
    param_draw(i_draw,:) = [rrho' llambda ssigma2];
    
    if ~isempty(save_file{1})
        if ~save_file{3}
            YYT_star_draw(i_draw,:) = YY_star(end,:);       
        else
            YY_star_draw(i_draw,:,:) = YY_star;
            mm_y0_draw(i_draw,:) = mm_y0;
            vv_y0_draw(i_draw,:) = vv_y0;
        end        
    end
end

%% Forecast evaluation
fcst_eval;
    
%% Common parameter estimates
param_est = reshape([mean(param_draw(ix_out,:)); ...
    quantile(param_draw(ix_out,:),[.05 .95]')],1,[]);

%% Save MCMC details if needed
if ~isempty(save_file{1}) 
    if ~save_file{3}
        parsave([save_file{1} save_file{2} '.mat'],param_draw,YYT_star_draw)        
    else
        parsave([save_file{1} save_file{2} '.mat'],param_draw,YY_star_draw, ...
            mm_y0_draw,vv_y0_draw)
    end
end

warning(warn2err);
warning(warn2err2);