# Forecasting with a Panel Tobit Model
# Laura Liu, Hyungsik Roger Moon, and Frank Schorfheide

# Load packages
library(moments)

# Set working directory and output paths
# https://stackoverflow.com/a/15373917
thisFile = function() {
  cmdArgs = commandArgs(trailingOnly = FALSE)
  needle = "--file="
  match = grep(needle, cmdArgs)
  if (length(match) > 0) { # Rscript
    return(normalizePath(sub(needle, "", cmdArgs[match])))
  } else { # 'source'd via R console
    return(normalizePath(sys.frames()[[1]]$ofile))
  }
}

setwd(dirname(dirname(thisFile())))
# If error occurred in setting the working directory automatically, please set it manually to 
# the "ReplicationFiles" root directory by setwd().

dir.create("application/results", showWarnings = FALSE)

data.dir = "application/data/"
result.dir = "application/results/"

# Specify the dataset
yr0 = 1994
qtr0 = 1
yr1 = 2018
qtr1 = 4
T_all = (yr1-yr0)*4+(qtr1-qtr0+1)
tag0 = paste0("_", as.character(yr0), "Q", as.character(qtr0), 
              "_", as.character(yr1), "Q", as.character(qtr1))

# ==================================================================================================
# Read data
# NOTE: not needed as the cleaned-up dataset is provided

# --------------------------------------------------------------------------------------------------
# big.data.dir = "application/bigdata/" # Not provided as the files are too big
# vars = scan(paste0(data.dir, "vars.txt"), what = "")
# vars = c(vars, "RCON2170")
# vars_chargeoff = vars[1:17]
# vars_loan = vars[18:30]
# 
# for (t in 1:T_all) {
#   
#   yr = (t+qtr0-2)%/%4+yr0
#   qtr = (t+qtr0-2)%%4+1
#   
#   yyyymm = yr*100+qtr*3
#   yymm = substr(as.character(yyyymm), start = 3, stop = 6)
#   print(yymm)
#   
#   # Commercial bank data
#   load(paste0(big.data.dir, "CALL", yymm, ".RData"))
#   
#   temp_q[, vars[!(vars %in% colnames(temp_q))]] = NA
#   
#   temp_q = subset(temp_q,
#                   select = c(RIAD4645, RIAD4638,
#                              RIADC891, RIADC893,
#                              RIAD3588,
#                              RIADC895, RIADC897,
#                              RIAD5411,
#                              RIADC234, RIADC235,
#                              RIADB514,
#                              RIADK129, RIADK205,
#                              
#                              RIAD4079, RIAD4107,
#                              RIAD4340,
#                              RIAD4093,
#                              
#                              RCON3387,
#                              RCONF158, RCONF159,
#                              RCON1460,
#                              RCON1480,
#                              RCON1797,
#                              RCON5367, RCON5368,
#                              RCONB561,
#                              RCONB562,
#                              
#                              RCON3210,
#                              RCON3360,
#                              RCON3123,
#                              
#                              RSSDID, RCON2170))
#   
#   temp_q =  temp_q[!duplicated(temp_q$RSSDID, fromLast = TRUE), ]
#   
#   # Chargeoffs: NA ~> 0
#   aux_q = temp_q[, vars_chargeoff]
#   aux_q[is.na(aux_q)] = 0
#   temp_q[, vars_chargeoff] = aux_q
#   
#   temp_q$yyyy = yr
#   temp_q$q = qtr
#   temp_q$y.q = yr+(qtr-1)/4
#   
#   temp_q_aux = temp_q
#   
#   if (t >= 2) {
#     
#     # Loans: previous quarter
#     id_q = unique(temp_q$RSSDID)
#     id_q0 = unique(temp_q0$RSSDID)
#     
#     for (ind_id in id_q) {
#       if (!is.element(ind_id, id_q0)) {
#         temp_q[temp_q$RSSDID==ind_id, vars_loan] = NA
#       } else {
#         temp_q[temp_q$RSSDID==ind_id, vars_loan] = temp_q0[temp_q0$RSSDID==ind_id, vars_loan]
#       }
#     }
#     
#     # Chargeoffs: ytd data ~> flow data
#     if (qtr > 1) {
#       for (ind_id in id_q) {
#         if (!is.element(ind_id, id_q0)) {
#           temp_q[temp_q$RSSDID==ind_id, vars_chargeoff] = NA
#         } else {
#           temp_q[temp_q$RSSDID==ind_id, vars_chargeoff] =
#             temp_q[temp_q$RSSDID==ind_id, vars_chargeoff]-temp_q0[temp_q0$RSSDID==ind_id, 
#                                                                   vars_chargeoff]
#         }
#       }
#     }
#     
#     if (qtr == 1 | t == 2) {
#       temp_y = temp_q
#     } else {
#       temp_y = rbind(temp_y, temp_q)
#     }
#     
#   }
#   
#   temp_q0 = temp_q_aux
#   
#   if (qtr == 4 | t == T_all) {
#     
#     ci_num = ifelse(is.na(temp_y$RIAD4645) & !is.na(temp_y$RIAD4638), 
#                     temp_y$RIAD4638, temp_y$RIAD4645)
#     
#     temp_y$ci = ci_num/temp_y$RCON3387*400
#     temp_y$cld = (temp_y$RIADC891+temp_y$RIADC893)/(temp_y$RCONF158+temp_y$RCONF159)*400
#     temp_y$mf = temp_y$RIAD3588/temp_y$RCON1460*400
#     temp_y$cre = (temp_y$RIADC895+temp_y$RIADC897)/temp_y$RCON1480*400
#     temp_y$hlc = temp_y$RIAD5411/temp_y$RCON1797*400
#     temp_y$rre = (temp_y$RIADC234+temp_y$RIADC235)/(temp_y$RCON5367+temp_y$RCON5368)*400
#     temp_y$cc = temp_y$RIADB514/temp_y$RCONB561*400
#     temp_y$con = (temp_y$RIADK129+temp_y$RIADK205)/temp_y$RCONB562*400
#     
#     temp_y$capital2assets = temp_y$RCON3210/temp_y$RCON2170
#     temp_y$loan2assets = temp_y$RCON3360/temp_y$RCON2170
#     temp_y$alll2loan = temp_y$RCON3123/temp_y$RCON3360
#     temp_y$diversification = temp_y$RIAD4079/(temp_y$RIAD4079+temp_y$RIAD4107)
#     temp_y$ROA = temp_y$RIAD4340/temp_y$RCON2170
#     temp_y$OCA = temp_y$RIAD4093/temp_y$RCON2170
#     
#     # Bank location data
#     branch = read.csv(paste0(big.data.dir, "ALL_", as.character(yr), ".csv"))
#     names(branch) = toupper(names(branch))
#     branch = subset(branch, subset = (branch$RSSDID %in% unique(temp_y$RSSDID) 
#                                       & (branch$CNTRYNA == "United States" | branch$CNTRYNA =="")),
#                     select = c(RSSDID, STNUMBR, DEPSUMBR, STCNTY))
#     branch$DEPSUMBR = as.numeric(gsub(",", "", branch$DEPSUMBR))
#     
#     dep = tapply(branch$DEPSUMBR, list(branch$RSSDID,branch$STNUMBR), sum)
#     RSSDID = as.numeric(rownames(dep))
#     stcode = as.numeric(colnames(dep)[apply(dep,1,which.max)])
#     frac_in = apply(dep, 1, function(x) max(x, na.rm = TRUE))/
#       apply(dep, 1, function(x) sum(x, na.rm = TRUE))
#     loc = data.frame(RSSDID, stcode, frac_in)
#     loc = merge(loc, branch[!duplicated(branch$RSSDID), c(1,4)], by = "RSSDID") 
#     
#     temp_y = merge(temp_y, loc, by = "RSSDID")
#     
#     if (t <= 4) {
#       bank = temp_y
#     } else {
#       bank = rbind(bank, temp_y)
#     }
#   }
#   
# }
# 
# # Local econ condition data
# hpi_unemp_inc = read.table(paste0(data.dir, "hpi_unemp_inc.txt"), header = TRUE)
# 
# bank = merge(bank, hpi_unemp_inc, by = c("yyyy", "q", "stcode"))
# bank = bank[order(bank$yyyy, bank$q, bank$stcode, bank$RSSDID), ]
# save(bank, file = paste0(data.dir, "bank", tag0, ".RData"))
# --------------------------------------------------------------------------------------------------

load(file = paste0(data.dir, "bank", tag0, ".RData"))

# ==================================================================================================
# Sample selection

T2 = 12
T_smpl = (yr1-yr0)*4+qtr1-qtr0-T2+2-1
# Note: The first sample starts with t0 = period (yrt0,qtr0) + one quarter, because of the lagged 
#       loan size in the denominator

chargeoff_names = c("ci", "cld", "mf", "cre", "hlc", "rre", "cc", "con")
n_chargeoff = length(chargeoff_names)

n_output = matrix(NA, T_smpl*n_chargeoff, 8)
colnames(n_output) = c("charge-off", "t0", paste0("n", c(0:5)))

stats_output = matrix(NA, T_smpl*n_chargeoff, 12)
colnames(stats_output) = c("charge-off", "t0", "%0", "mean", "min", "q25", "median", "q75", "max", 
                           "std", "skew", "kurt")

# Auxiliary function for step 5
step5 = function(x) {
  aux_std = sd(sort(x)[1:(T2-2)])
  aux_diff = abs(diff(x));
  isselected = sum((c(0, aux_diff)+c(aux_diff, 0)) > 10*aux_std) == 0
  return(isselected)
}

# Aggregate data for standardizing covariates
dagg = read.table(paste0(data.dir, "dagg_covariates.txt"))
colnames(dagg) = c("dhpi", "dunemp", "dinc")

for (t_smpl in 1:T_smpl) {
  
  yr = (t_smpl+1+qtr0-2)%/%4+yr0
  qtr = (t_smpl+1+qtr0-2)%%4+1
  t0_yq = yr+(qtr-1)/4
  
  data = bank[(bank$y.q>=t0_yq-0.01)&(bank$y.q<=t0_yq+(T2-1)/4+0.01),
              c("RSSDID", "RCON2170", "y.q", chargeoff_names, "dhpi", "dunemp", "dinc")]
  n0 = length(unique(data$RSSDID))
  
  # Step 1: Eliminate banks for which total assets are missing for all time periods in the sample.
  data = data[is.finite(data$RCON2170), ]
  n1 = length(unique(data$RSSDID))
  
  # Step 2: Compute avg non-missing total assets and eliminate banks with avg assets >= $1B.
  aux2 = aggregate(RCON2170~RSSDID, data = data, function(x) mean(x, na.rm = TRUE))
  ix2 = aux2$RCON2170 < 10^6
  id2 = aux2[as.logical(ix2), 1]
  data = data[data$RSSDID%in%id2, ]
  n2 = sum(ix2)
  
  # Standardize covariates
  ix = t_smpl+(0:(T2-2))
  data$dhpi = (data$dhpi-mean(dagg$dhpi[ix]))/sd(dagg$dhpi[ix])
  data$dunemp = (data$dunemp-mean(dagg$dunemp[ix]))/sd(dagg$dunemp[ix])
  data$dinc = (data$dinc-mean(dagg$dinc[ix]))/sd(dagg$dinc[ix])
  
  # Loop over loan categories
  for (i_chargeoff in 1:n_chargeoff) {
    print(c(t0_yq, chargeoff_names[i_chargeoff]))
    
    data_chargeoff = data[, c("RSSDID", "y.q", chargeoff_names[i_chargeoff], "dhpi", "dunemp", 
                              "dinc")]
    colnames(data_chargeoff)[3] = "chargeoff"
    ix = (i_chargeoff-1)*T_smpl+t_smpl
    stats_output[ix, 1:2] = c(chargeoff_names[i_chargeoff], t0_yq)
    n_output[ix, 1:5] = c(chargeoff_names[i_chargeoff], t0_yq, n0, n1, n2)
    
    # Step 3: Eliminate banks for which the target charge-off rate is missing (or infinite)
    #         for at least one period of the sample.
    if (sum(is.finite(data_chargeoff$chargeoff)) == 0) {
      next
    }
    aux3 = aggregate(chargeoff~RSSDID, data = data_chargeoff, function(x) sum(is.finite(x)))
    ix3 = aux3$chargeoff == T2
    id3 = aux3[as.logical(ix3), 1]
    if (length(id3) == 0) {
      next
    }
    data_chargeoff = data_chargeoff[data_chargeoff$RSSDID%in%id3, ]
    n3 = sum(ix3)
    
    # Step 4: Eliminate banks for which the specific charge-off rate is negative or > 400% 
    #         for at least one period of the sample.
    aux4 = aggregate(chargeoff~RSSDID, data = data_chargeoff, function(x) sum(x>=0 & x<=400))
    ix4 = aux4$chargeoff == T2
    id4 = aux4[as.logical(ix4), 1]
    if (length(id4) == 0) {
      next
    }
    data_chargeoff = data_chargeoff[data_chargeoff$RSSDID%in%id4, ]
    n4 = sum(ix4)
    
    # Step 5: First, for each bank, drop the two largest observations and calculate the stdd
    #         of the remaining observations.  
    #         Then, eliminate a bank if any successive change > 10 stdd.
    data_chargeoff = data_chargeoff[order(data_chargeoff$y.q), ]
    aux5 = aggregate(chargeoff~RSSDID, data = data_chargeoff, step5)
    ix5 = aux5$chargeoff
    id5 = aux5[as.logical(ix5), 1]
    if (length(id5) == 0) {
      next
    }
    data_chargeoff = data_chargeoff[data_chargeoff$RSSDID%in%id5, ]
    n5 = sum(ix5)
    
    # Save selected sample
    write.table(data_chargeoff[, c("RSSDID", "y.q", "chargeoff", "dhpi", "dunemp", "dinc")],
                file = paste0(data.dir, "t0_", as.character(yr), "Q", as.character(qtr), "_", 
                              chargeoff_names[i_chargeoff], ".txt"),
                row.names = FALSE, col.names = FALSE)
    
    # Output: sample sizes after selection steps and summary statistics
    n_output[ix, 6:8] = c(n3, n4, n5)
    
    stats_output[ix, 3] = mean(data_chargeoff$chargeoff==0)
    stats_output[ix, 4] = mean(data_chargeoff$chargeoff)
    stats_output[ix, 5:9] = quantile(data_chargeoff$chargeoff)
    stats_output[ix, 10] = sd(data_chargeoff$chargeoff)
    stats_output[ix, 11] = skewness(data_chargeoff$chargeoff)
    stats_output[ix, 12] = kurtosis(data_chargeoff$chargeoff)
  }
}

stats_output_aux = stats_output
stats_output_aux[, 3:12] = round(as.numeric(stats_output_aux[, 3:12]), digits = 2)

write.csv(n_output, file = paste0(result.dir, "n_output.csv"), row.names = FALSE)
write.csv(stats_output_aux, file = paste0(result.dir, "stats_output.csv"), row.names = FALSE)

# ==================================================================================================
# Bank characteristics for for selected samples
mat_smpl = matrix(c(2007, 2, 6, 2007, 2, 7), nrow = 3) # selected samples, ordered by time
n_smpl = ncol(mat_smpl)
if (n_smpl == 1) {
  is_new_time = TRUE
} else if (n_smpl == 2) {
  is_new_time = c(TRUE, sum(abs(mat_smpl[1:2, -1]-mat_smpl[1:2, -n_smpl]))>0)
} else {
  is_new_time = c(TRUE, colSums(abs(mat_smpl[1:2, -1]-mat_smpl[1:2, -n_smpl]))>0)
}

for (i_smpl in 1:n_smpl) {
  
  yr = mat_smpl[1, i_smpl]
  qtr = mat_smpl[2, i_smpl]
  i_chargeoff = mat_smpl[3, i_smpl]
  
  if (is_new_time[i_smpl]) {
    
    t0_yq = yr+(qtr-1)/4
    
    # Bank characteristics at t0
    data = bank[(bank$y.q>=t0_yq-0.01)&(bank$y.q<=t0_yq+0.01), ]
    data$ci_loan = data$RCON3387
    data$cld_loan = data$RCONF158+data$RCONF159
    data$mf_loan = data$RCON1460
    data$cre_loan = data$RCON1480
    data$hlc_loan = data$RCON1797
    data$rre_loan = data$RCON5367+data$RCON5368
    data$cc_loan = data$RCONB561
    data$con_loan = data$RCONB562
    
    data_bankvar = data[, c("RSSDID", colnames(data)[ncol(data)-n_chargeoff+(1:n_chargeoff)],
                            "capital2assets", "loan2assets", "alll2loan", "diversification", "ROA", 
                            "OCA")]
    
  }
  
  # Load selected sample
  data_chargeoff = 
    read.table(file = paste0(data.dir, "t0_", as.character(yr), "Q", as.character(qtr), "_", 
                             chargeoff_names[i_chargeoff], ".txt"),
               col.names = c("RSSDID", "y.q", "chargeoff", "dhpi", "dunemp", "dinc"))
  data_chargeoff_t0 = 
    data_chargeoff[(data_chargeoff$y.q>=t0_yq-0.01)&(data_chargeoff$y.q<=t0_yq+0.01), ]
  
  # Save bank characteristics for selected banks
  data_out = data_bankvar[data_bankvar$RSSDID %in% data_chargeoff_t0$RSSDID, ]
  write.table(data_out[order(data_out$RSSDID), ], 
              file = paste0(data.dir, "t0_", as.character(yr), "Q", as.character(qtr), "_", 
                            chargeoff_names[i_chargeoff], "_bankvar.txt"), 
              row.names = FALSE, col.names = FALSE, na = "nan")
}
