% Generate graphs: Effects (Term I and II) of HPI and UR
% (for selected samples: v_smpl0 = RRE and CC samples starting at 2007Q2)

%% Prepare
n_X = length(ix_X);

% Rescale units of all covariates to be 1%
dagg = importdata([data_path 'dagg_covariates.txt']); % aggregate data for standardizing covariates
v_unit = [100 1 25];

% adverse scenarios
X_dev = [-5 5 -5]; % decrease housing price inflation rate by 5%
                   % increase unemployment rate by 5%
                   % decrease personal income growth rate by 5%
X_devsign = sign(X_dev);

n_shock = 10; % number of draws of future shocks for each MCMC draw of parameters

%% Setup style
graph_size = [6 4];
lgd = [];
SFont = 16;

n_term = 2; % terms I and II

color_line = {'b','k'};
color_shade = [.7 .7 1; .7 .7 .7];

i_est = 1; % Flexible CRE, heterosk.
is_load_est = 1; % load estimation results

%% Loop over samples
for i_smpl0 = 1:n_smpl0
    %% Prepare: sample i and forecast
    sub_smpl_i; % subroutine: prepare sample i
    sub_fcst_prep; % subroutine: "Mean" and "StdD" of the trunc normal pred dist for each MCMC draw
    
    % Rescale units of all covariates to be 1%
    unit_aux = std(dagg(t_smpl+(0:T),ix_X)).*v_unit(ix_X);
    param_rescaled = est_draw.param_draw;
    param_rescaled(:,2:n_X+1) = param_rescaled(:,2:n_X+1)./unit_aux;
    
    % "Mean" of the trunc normal pred dist for each MCMC draw (at x+dx)
    m_draw1 = repmat(m_draw,[1,1,n_X]);
    for i_X = 1:n_X
        m_draw1(:,:,i_X) = m_draw1(:,:,i_X)...
            +X_dev(ix_X(i_X))*param_rescaled(:,ix_X(i_X)+1);
    end
    
    %% Calculate treatment effects
    % Simulate future realizations
    u_draw = randn(N_draw*n_shock,N);
    Y1star_draw = u_draw.*repmat(s_draw(ix_out,:),[n_shock,1]) ...
        +repmat(m_draw(ix_out,:),[n_shock,1]); % at orginal x
    Y1star_draw1 = u_draw.*repmat(s_draw(ix_out,:),[n_shock,1]) ...
        +repmat(m_draw1(ix_out,:,:),[n_shock,1,1]); % at x+dx
    
    % Unit-specific posterior means of treatment effects, two terms
    mean_ME = nan(n_X,N,n_term);
    mean_ME(:,:,1) = permute(mean(Y1star_draw1.*((Y1star_draw1>0)-(Y1star_draw>0))),[3,2,1]) ...
        ./X_dev(ix_X)';
    mean_ME(:,:,2) = ...
        permute(mean(permute(repmat(param_rescaled(ix_out,ix_X+1),[n_shock,1]),[1,3,2]) ...
                     .*(Y1star_draw>0)),[3,2,1]);
    
    % Unit-specific 90% credible intervals of treatment effects, two terms
    q_ME = nan(n_X,N,2,n_term);
    q_ME(:,:,:,1) = permute(quantile(Y1star_draw1.*((Y1star_draw1>0)-(Y1star_draw>0)) ...
                                     ./permute(X_dev(ix_X),[1,3,2]),[.05 .95]),[3,2,1]); 
    q_ME(:,:,:,2) = ...
        permute(quantile(permute(repmat(param_rescaled(ix_out,ix_X+1),[n_shock,1]),[1,3,2]) ...
                         .*(Y1star_draw>0),[.05 .95]),[3,2,1]);
    
    %% Plot graph
    % Sort units by E[\lambda_i/\sigma_i|data]
    [~,ix_ME] = sort(mean(est_draw.het_draw(ix_out,:,1)./sqrt(est_draw.het_draw(ix_out,:,2))));
    for i_X = 1:n_X
        mean_aux = permute(mean_ME(i_X,ix_ME,:),[2,3,1])*X_devsign(ix_X(i_X));
        q_aux = permute(q_ME(i_X,ix_ME,:,:),[2,3,4,1])*X_devsign(ix_X(i_X));
        
        F1 = figure;
        % Unit-specific 90% credible intervals of treatment effects
        for i_term = n_term:-1:1 % term I in the top layer
            jbfill(1:N,q_aux(:,2,i_term)',q_aux(:,1,i_term)', ...
                color_shade(i_term,:),color_shade(i_term,:),1,0.5);       
        end
        
        % Unit-specific posterior means of treatment effects
        hold on
        for i_term = n_term:-1:1 % term I in the top layer         
            plot(mean_aux(:,i_term),color_line{i_term},'linewidth',2);            
        end  
        hold off
        
        xlim([1 N])
            
        % Save graph
        graph_tag = [result_path spec_smpl '_ME_x' num2str(ix_X(i_X))];
        graph_out(F1,graph_tag,[],SFont,graph_size,lgd);
    end
end

%% Close files
close all