% Generate tables: parameter estimates and set/density forecast performance
% (for selected samples: v_smpl0 = RRE and CC samples starting at 2007Q2)

%% Prepare: rescale units of all covariates to be 1%
dagg = importdata([data_path 'dagg_covariates.txt']); % aggregate data for standardizing covariates
v_unit = [100 1 25];

%% Setup style
est_print = ... % row labels in output tables
   {'Flexible CRE, Heterosk.  ';
    'Normal CRE, Heterosk.    ';
    'Flexible RE, Heterosk.   ';
    'Normal RE, Heterosk.     ';
    'Flexible CRE, Homosk.    ';
    'Normal CRE, Homosk.      ';
    'Flexible RE, Homosk.     ';
    'Normal RE, Homosk.       ';
    'Pooled Tobit, Homosk.    ';
    'Pooled linear, Homosk.   '}; 

file_names = {'param','sfcstPtwise','sfcstAvg','dfcst'};
n_file = length(file_names);
file_id = nan(1,n_file);
for i_file = 1:n_file
    file_id(i_file) = fopen([result_path spec '_' file_names{i_file} '.txt'],'w'); 
    % change to 'w'/'a' if needed
end
ncol_file = [(n_param-2)*3 n_sfcst n_sfcst n_dfcst]';

is_load_est = 0; % not load estimation results

%% Loop over samples
for i_smpl0 = 1:n_smpl0
    %% Prepare: sample i
    sub_smpl_i; % subroutine: prepare sample i
    
    %% Set / density forecst evaluation
    table_sfcstPtwise = cell2mat(sfcstPtwise_stat(:,i_smpl));
    table_sfcstAvg = cell2mat(sfcstAvg_stat(:,i_smpl));
    table_dfcst = cell2mat(dfcst_stat(:,i_smpl));
    
    %% Parameter estimation
    table_param = cell2mat(param_est(:,i_smpl));  
    table_param(:,3+(1:3*(n_param-3))) =  table_param(:,3+(1:3*(n_param-3))) ...
        ./repelem(std(dagg(t_smpl+(0:T),ix_X)).*v_unit(ix_X),1,3);
        % rescale units of all covariates to be 1%
    
    %% Generate output tables
    table_out = {table_param(:,1:(n_param-2)*3),table_sfcstPtwise,table_sfcstAvg, ...
        table_dfcst}; 
        % param order: [Mean, Conf. Interval] x [AR1, D(lnHPI), D(UR), D(lnINC)]
        % sfcst order: [Cover. Freq., Ave. Len., Empty Set, {0}, [0,b], {0}U[0,b], Other]
        % dfcst order: [LPS, CRPS]

    for i_file = 1:n_file
        fprintf(file_id(i_file),...
            [num2str(yr) 'Q' num2str(qtr) ' ' chargeoff_names{i_chargeoff} '\r\n']);
        
        for i_est = 1:n_est
            if i_file == 1 % parameter estimates with confidence intervals
                fprintf(file_id(i_file), ...
                    ['%4.25s ' repmat('& %7.3f & [%7.3f,%7.3f]',1,n_param-2) ' %4.6s\r\n'], ...
                    est_print{i_est},table_out{i_file}(i_est,:),'\\ ');
            else % other tables
                fprintf(file_id(i_file), ...
                    ['%4.25s ' repmat('& %7.3f ',1,ncol_file(i_file)) ' %4.6s\r\n'], ...
                    est_print{i_est},table_out{i_file}(i_est,:),'\\ ');
            end
        end
    
        fprintf(file_id(i_file),'\r\n');
    end
end

%% Close files
for i_file = 1:n_file
    fclose(file_id(i_file));
end