function [sfcstPtwise_stat, sfcstAvg_stat, dfcst_stat, param_est] = ...
    pool_tobit(Y,draws,param0,save_file)
%% Input: Y
YY = Y(1:end-1,:);
[T1,N] = size(YY);
T = T1-1;
ix0 = find(Y(end,:)==0);
ix1 = find(Y(end,:)~=0);

%% Input: X
n_coef = 1;

%% Input: numbers of draws
B_draw = draws(1);
N_draw = draws(2);

%% Input: tuning parameters for prior
ttau = param0{2};

%% Prepare: prior parameters 
% p(y*_i0)
m_my0_0 = 0;
v_my0_0 = ttau;
a_vy0_0 = 3;
b_vy0_0 = 2;

% rho, sigma2
M_coef0 = zeros(n_coef+1,1);
V_coef0 = ttau*eye(n_coef+1);
a_ssigma20 = 3;
b_ssigma20 = mean(var(YY,1))*(a_ssigma20-1);

%% Prepare: initial values 
% y*_it
YY_star = YY;

% rho, lambda, sigma2
Ys = YY_star(2:end,:);
Xs(:,:,1) = YY_star(1:end-1,:);

vecX = [reshape(Xs,N*T,n_coef) ones(N*T,1)];
M_coef1 = (vecX'*vecX)\(vecX'*Ys(:));
rrho = M_coef1(1:n_coef);
llambda = M_coef1(end);
ssigma2 = var(Ys(:)-vecX*M_coef1);

%% Prepare: save draws
m_draw = nan(B_draw+N_draw,N);
s_draw = nan(B_draw+N_draw,N);

param_draw = nan(B_draw+N_draw,n_coef+2);

%% Prepare: segment structure
seg_start = cell(1,N);
seg_end = cell(1,N);
n_seg = nan(1,N);
for i = 1:N
    seg_all = diff(YY(:,i)==0);
    seg_start{i} = find(seg_all==1)+1;
    seg_end{i} = find(seg_all==-1);
    if YY(1,i)==0
        seg_start{i} = [1; seg_start{i}];
    end
    if YY(end,i)==0
        seg_end{i} = [seg_end{i}; T1];
    end
    n_seg(i) = length(seg_start{i});
end

%% Gibbs sampler
for i_draw = 1:(B_draw+N_draw)
    %% Display iteration
    if mod(i_draw,100) == 0
        disp([save_file{2} ': i_draw = ' num2str(i_draw)])
    end
    
    %% Draw p(y*_i0)    
    v_my0_1 = 1/(N+1/v_my0_0);
    m_my0_1 = (sum(YY_star(1,:))+m_my0_0/v_my0_0)*v_my0_1;
    a_vy0_1 = a_vy0_0+N/2;
    b_vy0_1 = b_vy0_0+(sum(YY_star(1,:).^2)+m_my0_0^2/v_my0_0-m_my0_1^2/v_my0_1)/2;
    vv_y0 = 1/gamrnd(a_vy0_1,1/b_vy0_1);
    mm_y0 = sqrt(v_my0_1*vv_y0)*randn(1)+m_my0_1;
    
    %% Draw y*_it     
    QQ_T1 = diag((1-rrho(1).^((1:T1)*2))/(1-rrho(1)^2));
    for t = 1:T1
        QQ_T1(t,(t+1):T1) = rrho(1).^(1:(T1-t))*QQ_T1(t,t);
        QQ_T1((t+1):T1,t) = QQ_T1(t,(t+1):T1);
    end
    
    QQ_T1_1 = diag([0 (1-rrho(1).^((1:(T1-1))*2))/(1-rrho(1)^2)] ...
        +vv_y0/ssigma2*rrho(1).^((0:(T1-1))*2)); % for y_i0 = 0
    for t = 1:T1
        QQ_T1_1(t,(t+1):T1) = rrho(1).^(1:(T1-t))*QQ_T1_1(t,t);
        QQ_T1_1((t+1):T1,t) = QQ_T1_1(t,(t+1):T1);
    end
            
    for i = 1:N
        if n_seg(i) == 0
            continue
        end
        for i_seg = 1:n_seg(i)
            
            if seg_end{i}(i_seg) == T1
                TT = seg_end{i}(i_seg)-seg_start{i}(i_seg)+1;
            else
                TT = seg_end{i}(i_seg)-seg_start{i}(i_seg)+2;
            end
            
            
            if seg_start{i}(i_seg) == 1
                AA = (1-rrho(1).^((1:TT)-1)')/(1-rrho(1))*llambda+rrho(1).^(0:(TT-1))'*mm_y0;
                QQ = QQ_T1_1(1:TT,1:TT);
            else                
                AA = (1-rrho(1).^(1:TT)')/(1-rrho(1))*llambda ...
                    +rrho(1).^(1:TT)'*YY(seg_start{i}(i_seg)-1,i);
                QQ = QQ_T1(1:TT,1:TT);
            end
            
            if seg_end{i}(i_seg) ~= T1
                AA = AA(1:(TT-1))+QQ(1:(TT-1),TT)/QQ(TT,TT)*(YY(seg_end{i}(i_seg)+1,i)-AA(TT));
                QQ = QQ(1:(TT-1),1:(TT-1))-QQ(1:(TT-1),TT)*QQ(TT,1:(TT-1))/QQ(TT,TT);
                TT = TT-1;
            end
            
            if TT == 1
                YY_star(seg_start{i}(i_seg),i) = AA+sqrt(QQ)*trandn(-inf,-AA/sqrt(QQ));
            else
                for t = 1:TT
                    aux_A = AA;
                    aux_A(t) = [];
                    aux_Ystar = YY_star(seg_start{i}(i_seg):seg_end{i}(i_seg),i);
                    aux_Ystar(t) = [];
                    
                    aux_Q = QQ;
                    aux_Q(t,:) = [];
                    aux_v = aux_Q(:,t);
                    aux_Q(:,t) = [];
                    
                    try
                        aux_m = AA(t)+(aux_v'/aux_Q)*(aux_Ystar-aux_A);
                    catch ME
                        disp([save_file{2} ': i_draw = ' num2str(i_draw) ', i = ' num2str(i) ...
                            ', i_seg = ' num2str(i_seg) ', t = ' num2str(t)])
%                         disp(ME.message);
                        disp('YY_seg = ');
                        disp(YY(seg_start{i}(i_seg):seg_end{i}(i_seg),i)')
                        disp('YY_star_seg = ');
                        disp(YY_star(seg_start{i}(i_seg):seg_end{i}(i_seg),i)')
                        disp('aux_Q = ');
                        disp(aux_Q)
                        disp('eig(aux_Q) = ');
                        disp(eig(aux_Q)');
                        rethrow(ME)
                    end
                    aux_s = sqrt(QQ(t,t)-(aux_v'/aux_Q)*aux_v)*sqrt(ssigma2);
                    YY_star(seg_start{i}(i_seg)-1+t,i) = aux_m+aux_s*trandn(-inf,-aux_m/aux_s);
                end
            end
        end
    end
    
    %% Draw rho, lambda, sigma2 
	Ys = YY_star(2:end,:);
    Xs(:,:,1) = YY_star(1:end-1,:);
 
    vecX = [reshape(Xs,N*T,n_coef) ones(N*T,1)];
    V_coef1 = eye(n_coef+1)/(vecX'*vecX/ssigma2+eye(n_coef+1)/V_coef0);
    M_coef1 = V_coef1*(vecX'*Ys(:)/ssigma2+V_coef0\M_coef0);
    coef1 = chol(V_coef1)'*randn(n_coef+1,1)+M_coef1;
    rrho = coef1(1:n_coef);
    llambda = coef1(end);
    
    a_ssigma21 = a_ssigma20+N*T/2;	    
    b_ssigma21 = b_ssigma20+(Ys(:)-vecX*coef1)'*(Ys(:)-vecX*coef1)/2;
    ssigma2 = 1/gamrnd(a_ssigma21,1/b_ssigma21);
    
    %% Prepare: forecast evaluation
    m_draw(i_draw,:) = YY_star(end,:)*rrho+llambda;
    s_draw(i_draw,:) = sqrt(ssigma2);
    
    %% Save draws
    param_draw(i_draw,:) = [rrho' llambda ssigma2];    
end

%% Forecast evaluation
fcst_eval;

%% Common parameter estimates
param_est = reshape([mean(param_draw(ix_out,:)); quantile(param_draw(ix_out,:),[.05 .95])],1,[]);