% Generate graphs: Convergence Diagnostics Based on rho^(j) Sequence

%% Setup style
graph_size = [6 4];
SFont = 16;
lgd = [];

coef_names = {'rho'};
i_coef = 1;

fig_names = {'trace','acf','post'};
n_fig = length(fig_names);
F1 = cell(n_fig,n_dgp);
het_cell = cell(1,n_dgp);

%% Load data, overall results, and estimator-specific results
for i_dgp = 1:n_dgp
    spec = ['dgp' num2str(i_dgp)];
    load([temp_path spec '.mat'])
    
    i_est = 1;
    est_draw = matfile([temp_path spec '_isim1_' est_names{i_est} '.mat']);
    het_cell{1,i_dgp} = est_draw.param_draw;
end

%% Prepare for graphs
ix_out = B_draw+(1:N_draw);
het_mat = cell2mat(het_cell);
draws_lim = [.75 max(het_mat(:))];
dens_max = nan(1,n_dgp);

%% Plot graphs
for i_dgp = 1:n_dgp
    aux_draw = het_mat(:,i_dgp);
    for i_fig = 1:n_fig
        F1{i_fig,i_dgp} = figure;
        if i_fig == 1 % trace plot
            hold on
            plot(1:B_draw+N_draw,aux_draw,'k')
            jbfill(B_draw+[1 N_draw],draws_lim(2)*ones(1,2),draws_lim(1)*ones(1,2), ...
                [.7 .7 .7],[.7 .7 .7],1,0.5);
            hold off
            xlim([1 B_draw+N_draw])
            ylim(draws_lim)
            yline(rrho,'k--')
            grid on
        elseif i_fig == 2 % autocorrelation
            [~,~,~,h] = autocorr(aux_draw(ix_out),50);
            h(1).Color = [0 0 0];
            delete(h(2:3));
            xlim([0 50])
            ylim([0 1])
            xlabel('')
            ylabel('')
            title('')
        else % posterior distribution
            [f1,xi1] = ksdensity(aux_draw(ix_out));
            dens_max(1,i_dgp) = max(f1); 
            plot(xi1,f1,'k','linewidth',2)
            xlim(draws_lim)
            xline(rrho,'k--')
        end
    end
end

%% Set common y-scale for posterior distribution and save graphs
dens_lim = [0 max(dens_max)];
for i_dgp = 1:n_dgp
    spec = ['dgp' num2str(i_dgp)];
    for i_fig = 1:n_fig
        figure(F1{i_fig,i_dgp});
        if i_fig == 3
            ylim(dens_lim)
        end
        graph_tag = [result_path spec '_' coef_names{i_coef} '_' fig_names{i_fig}];
        graph_out(F1{i_fig,i_dgp},graph_tag,[],SFont,graph_size,lgd);
    end
end

%% Close files
close all